@@ -1,4 +1,4 @@
-from gitlabform.gitlab.core import GitLabCore
+from gitlabform.gitlab.core import GitLabCore, NotFoundException
 
 
 class GitLabBranches(GitLabCore):
@@ -22,30 +22,24 @@ class GitLabBranches(GitLabCore):
         )
 
     # new API
-    def branch_access_level(
-        self,
-        project_and_group_name,
-        branch,
-        push_access_level,
-        merge_access_level,
-        unprotect_access_level,
-    ):
+    def branch_access_level(self, project_and_group_name, branch, protect_settings):
+
+        url = "projects/%s/protected_branches?name=%s"
+        parameters_list = [
+            project_and_group_name,
+            branch,
+        ]
         return self._make_requests_to_api(
-            "projects/%s/protected_branches?name=%s&push_access_level=%s&merge_access_level=%s&unprotect_access_level=%s",
-            (
-                project_and_group_name,
-                branch,
-                push_access_level,
-                merge_access_level,
-                unprotect_access_level,
-            ),
+            url,
+            tuple(parameters_list),
             method="POST",
-            data={},
+            # TODO: check why is 409 Conflict accepted here :/
             expected_codes=[
                 200,
                 201,
                 409,
-            ],  # TODO: check why is 409 Conflict accepted here :/
+            ],
+            json=protect_settings,
         )
 
     def branch_code_owner_approval_required(
@@ -108,6 +102,53 @@ class GitLabBranches(GitLabCore):
             "projects/%s/protected_branches/%s", (project_and_group_name, branch)
         )
 
+    def get_user_to_protect_branch(self, user_name):
+        return self._get_user_id(user_name)
+
+    def get_only_branch_access_levels(self, project_and_group_name, branch):
+        try:
+            result = self._make_requests_to_api(
+                "projects/%s/protected_branches/%s", (project_and_group_name, branch)
+            )
+
+            push_access_levels = set()
+            merge_access_levels = set()
+            push_access_user_ids = set()
+            merge_access_user_ids = set()
+            unprotect_access_level = None
+
+            if "push_access_levels" in result:
+                for push_access in result["push_access_levels"]:
+                    if not push_access["user_id"]:
+                        push_access_levels.add(push_access["access_level"])
+                    else:
+                        push_access_user_ids.add(push_access["user_id"])
+
+            if "merge_access_levels" in result:
+                for merge_access in result["merge_access_levels"]:
+                    if not merge_access["user_id"]:
+                        merge_access_levels.add(merge_access["access_level"])
+                    else:
+                        merge_access_user_ids.add(merge_access["user_id"])
+
+            if (
+                "unprotect_access_levels" in result
+                and len(result["unprotect_access_levels"]) == 1
+            ):
+                unprotect_access_level = result["unprotect_access_levels"][0][
+                    "access_level"
+                ]
+
+            return (
+                sorted(push_access_levels),
+                sorted(merge_access_levels),
+                sorted(push_access_user_ids),
+                sorted(merge_access_user_ids),
+                unprotect_access_level,
+            )
+        except NotFoundException:
+            return None, None, None, None, None
+
     def create_branch(
         self, project_and_group_name, new_branch_name, create_branch_from_ref
     ):
@@ -118,7 +159,7 @@ class GitLabBranches(GitLabCore):
         }
         self._make_requests_to_api(
             "projects/%s/repository/branches",
-            (project_and_group_name),
+            project_and_group_name,
             method="POST",
             data=data,
             expected_codes=[200, 201],
