@@ -1,9 +1,5 @@
 import functools
-import sys
 
-import cli_ui
-
-from gitlabform import EXIT_INVALID_INPUT
 from gitlabform.gitlab.core import GitLabCore, NotFoundException
 
 
@@ -32,14 +28,18 @@ class GitLabGroups(GitLabCore):
             for group in groups:
                 if group["full_path"].lower() == some_string.lower():
                     return group
-            raise NotFoundException
+            raise NotFoundException(
+                f"Group/subgroup with path '{some_string}' not found."
+            )
 
-    def create_group(self, name, path, visibility="private"):
+    def create_group(self, name, path, parent_id=None, visibility="private"):
         data = {
             "name": name,
             "path": path,
             "visibility": visibility,
         }
+        if parent_id is not None:
+            data["parent_id"] = parent_id
         return self._make_requests_to_api(
             "groups", data=data, method="POST", expected_codes=201
         )
@@ -66,6 +66,7 @@ class GitLabGroups(GitLabCore):
     def get_projects(self, group, include_archived=False):
         """
         :param group: group name
+        :param include_archived: set to True if archived projects should also be returned
         :return: sorted list of strings "group/project_name". Note that only projects from "group" namespace are
                  returned, so if "group" (= members of this group) is also a member of some projects, they won't be
                  returned here.
@@ -111,14 +112,6 @@ class GitLabGroups(GitLabCore):
             "groups/%s", project_and_group_name, "PUT", group_settings
         )
 
-    def delete_group_secret_variable(self, group, secret_variable_key):
-        self._make_requests_to_api(
-            "groups/%s/variables/%s",
-            (group, secret_variable_key),
-            method="DELETE",
-            expected_codes=[204, 404],
-        )
-
     def post_group_secret_variable(self, group, secret_variable):
         # secret_variable has to be like documented at:
         # https://docs.gitlab.com/ee/api/group_level_variables.html#create-variable
@@ -165,14 +158,7 @@ class GitLabGroups(GitLabCore):
     def add_share_to_group(
         self, group, share_with_group_name, group_access, expires_at=None
     ):
-        try:
-            share_with_group_id = self.get_group_id_case_insensitive(
-                share_with_group_name
-            )
-        except NotFoundException:
-            cli_ui.error(f"Group {share_with_group_name} not found.")
-            sys.exit(EXIT_INVALID_INPUT)
-
+        share_with_group_id = self.get_group_id_case_insensitive(share_with_group_name)
         data = {"group_id": share_with_group_id, "expires_at": expires_at}
         if group_access is not None:
             data["group_access"] = group_access
@@ -186,13 +172,7 @@ class GitLabGroups(GitLabCore):
         )
 
     def remove_share_from_group(self, group, share_with_group_name):
-        try:
-            share_with_group_id = self.get_group_id_case_insensitive(
-                share_with_group_name
-            )
-        except NotFoundException:
-            cli_ui.error(f"Group {share_with_group_name} not found.")
-            sys.exit(EXIT_INVALID_INPUT)
+        share_with_group_id = self.get_group_id_case_insensitive(share_with_group_name)
 
         # 404 means that the user is already removed, so let's accept it for idempotency
         return self._make_requests_to_api(
